@@ -28,7 +28,8 @@ sap.ui.define([
 
          let jsrp = EVE.JSR.source_dir;
          // take out 'jsrootsys' and replace it with 'rootui5sys/eve7/'
-         this.eve_path = jsrp.substring(0, jsrp.length - 10) + 'rootui5sys/eve7/';
+         this.top_path = jsrp.substring(0, jsrp.length - 10);
+         this.eve_path = this.top_path + 'rootui5sys/eve7/';
 
          this._logLevel = 1; // 0 - error, 1 - warning, 2 - info, 3 - debug
 
@@ -39,6 +40,15 @@ sap.ui.define([
 
          this._selection_map = {};
          this._selection_list = [];
+
+         this.initialMouseX = 0;
+         this.initialMouseY = 0;
+         this.lastOffsetX = 0;
+         this.lastOffsetY = 0;
+         this.firstMouseDown = true;
+         this.scale = false;
+         this.pickedOverlayObj;
+         this.initialSize = 0;
       }
 
       init(controller)
@@ -47,14 +57,6 @@ sap.ui.define([
 
          let pthis = this;
 
-         if (!datGUI) {
-            datGUI = new EVE.JSR.gui.GUI({ autoPlace: false, width: 220 });
-            datGUI.domElement.id = 'posDatGUI';
-            pthis.datGUIFolder = datGUI.addFolder("background")
-            let dome = this.controller.getView().getParent().getParent();
-            dome.getDomRef().appendChild(datGUI.domElement);
-         }
-
          // For offline mode, one needs a a full URL or the request
          // gets forwarded to openi5.hana.ondemand.com.
          // This has to be understood and fixed. Loading of shaders
@@ -76,17 +78,24 @@ sap.ui.define([
 
       bootstrap()
       {
-         this.creator = new EveElements(RC, this);
-         // this.creator.useIndexAsIs = EVE.JSR.decodeUrl().has('useindx');
-
          RC.GLManager.sCheckFrameBuffer = false;
          RC.Object3D.sDefaultPickable = false;
          RC.PickingShaderMaterial.DEFAULT_PICK_MODE = RC.PickingShaderMaterial.PICK_MODE.UINT;
 
          this.createRCoreRenderer();
+
+         this.creator = new EveElements(RC, this);
+         // this.creator.useIndexAsIs = EVE.JSR.decodeUrl().has('useindx');
+         if (this.RQ_Mode != "Direct") {
+            this.creator.SetupPointLineFacs(this.RQ_SSAA,
+                                            this.RQ_MarkerScale * this.canvas.pixelRatio,
+                                            this.RQ_LineScale   * this.canvas.pixelRatio);
+         }
+
          this.controller.createScenes();
          this.controller.redrawScenes();
          this.setupEventHandlers();
+         this.updateViewerAttributes();
 
          this.controller.glViewerInitDone();
       }
@@ -109,6 +118,10 @@ sap.ui.define([
       {
          return this.scene;
       }
+      get_overlay_scene()
+      {
+         return this.overlay_scene;
+      }
 
       //==============================================================================
 
@@ -130,30 +143,29 @@ sap.ui.define([
          this.renderer = new RC.MeshRenderer(this.canvas, RC.WEBGL2,
                                              { antialias: false, stencil: false });
          this.renderer._logLevel = 0;
-         this.renderer.clearColor = "#FFFFFF00"; // "#00000000";
          this.renderer.addShaderLoaderUrls(this.eve_path + RC.REveShaderPath);
          this.renderer.pickObject3D = true;
 
+         RC.Cache.enabled = true;
+         this.tex_cache = new RC.TextureCache;
+
          // add dat GUI option to set background
          let eveView = this.controller.mgr.GetElement(this.controller.eveViewerId);
-         let name = eveView.fName;
-         name = name.substring(0, 3);
-         let parName = name + "_WhiteBG";
-         let conf = {}; conf[parName] = true;
-         let pr = this;
-         if (this.controller.getView().getViewData()) {
-            datGUI.__folders.background.add(conf, parName).onChange(function (wbg) {
-               conf[parName] = wbg;
-               eveView.clearColor = wbg ? "#FFFFFF00" : "#00000000";
-               pr.renderer.clearColor = eveView.clearColor;
-               pr.request_render();
-            });
+         if (eveView.BlackBg)
+         {
+            this.bgCol =  new RC.Color(0,0,0);
+            this.fgCol = new RC.Color(1,1,1);
          }
-         else if (eveView.clearColor) {
-            pr.renderer.clearColor = eveView.clearColor;
+         else
+         {
+            this.bgCol = new RC.Color(1,1,1);
+            this.fgCol = new RC.Color(0,0,0);
          }
 
+         // always use black clear color except in tone map
+         this.renderer.clearColor = "#00000000";
          this.scene = new RC.Scene();
+         this.overlay_scene = new RC.Scene();
 
          this.lights = new RC.Group;
          this.lights.name = "Light container";
@@ -166,6 +178,12 @@ sap.ui.define([
          // let light_3d_ctor = function(col, int, dist, decay, args) { return new RC.DirectionalLight(col, int); };
          let light_2d_ctor = function(col, int) { return new RC.DirectionalLight(col, int); };
 
+         // guides
+         this.axis = new RC.Group();
+         this.axis.name = "Axis";
+         // this.overlay_scene.add(this.axis); // looks worse for now put to scene
+         this.scene.add(this.axis);
+
          if (this.controller.isEveCameraPerspective())
          {
             this.camera = new RC.PerspectiveCamera(75, w / h, 20, 4000);
@@ -223,7 +241,7 @@ sap.ui.define([
 
          this.rot_center = new RC.Vector3(0,0,0);
 
-         this.rqt = new RC.RendeQuTor(this.renderer, this.scene, this.camera);
+         this.rqt = new RC.RendeQuTor(this.renderer, this.scene, this.camera, this.overlay_scene);
          if (this.RQ_Mode == "Direct")
          {
             this.rqt.initDirectToScreen();
@@ -231,18 +249,23 @@ sap.ui.define([
          else if (this.RQ_Mode == "Simple")
          {
             this.rqt.initSimple(this.RQ_SSAA);
-            this.creator.SetupPointLineFacs(this.RQ_SSAA,
-                                            this.RQ_MarkerScale * this.canvas.pixelRatio,
-                                            this.RQ_LineScale   * this.canvas.pixelRatio);
          }
          else
          {
             this.rqt.initFull(this.RQ_SSAA);
-            this.creator.SetupPointLineFacs(this.RQ_SSAA,
-                                            this.RQ_MarkerScale * this.canvas.pixelRatio,
-                                            this.RQ_LineScale   * this.canvas.pixelRatio);
          }
          this.rqt.updateViewport(w, h);
+
+
+         // AMT secondary selection bug workaround for RenderCore PR #21
+         this.rqt.pick_instance = function(state)
+         {
+            return this.pick_instance_low_level(this.pqueue, state);
+         }
+         this.rqt.pick_instance_overlay = function(state)
+         {
+            return this.pick_instance_low_level(this.ovlpqueue, state);
+         }
       }
 
       setupEventHandlers()
@@ -311,6 +334,23 @@ sap.ui.define([
             glc.resetRenderer();
          });
 
+         dome.addEventListener("mouseup", function() {
+            glc.handleOverlayMouseUp();
+         });
+
+         dome.addEventListener("mousedown", function(event) {
+            console.log("event mousedown", event);
+            if (event.button == 0 || event.button == 2)
+            {
+               glc.handleOverlayMouseDown(event);
+
+            }
+         });
+
+         dome.addEventListener("mousemove", function(event) {
+            glc.handleOverlayMouseMove(event);
+         });
+
          // Key-handlers go on window ...
 
          window.addEventListener('keydown', function(event) {
@@ -458,6 +498,109 @@ sap.ui.define([
          this.controls.update();
       }
 
+
+      updateViewerAttributes()
+      {
+         let eveView = this.controller.mgr.GetElement(this.controller.eveViewerId);
+         if (eveView.BlackBg)
+         {
+            this.fgCol = this.creator.ColorWhite;
+            this.bgCol = this.creator.ColorBlack;
+         }
+         else
+         {
+            this.bgCol = this.creator.ColorWhite;
+            this.fgCol = this.creator.ColorBlack;
+         }
+
+         this.axis.clear();
+         if (eveView.AxesType > 0)
+            this.makeAxis();
+
+         this.request_render();
+      }
+
+      makeAxis()
+      {
+         function formatFloat(val) {
+            let lg = Math.log10(Math.abs(val));
+            let fs = "undef";
+
+            if (lg < 0) {
+                if (lg > -1) {
+                    fs = val.toFixed(2);
+                }
+                else if (lg > -2) {
+                    fs = val.toFixed(3);
+                }
+                else {
+                    fs = val.toExponential(2);
+                }
+            }
+            else {
+                if (lg < 2)
+                    fs = val.toFixed(1);
+                else if (lg < 4)
+                    fs = Math.round(val);
+                else
+                    fs = val.toExponential(2);
+            }
+            return val > 0 ? "+" + fs : fs;
+         }
+
+         let bb = new RC.Box3();
+         bb.setFromObject(this.scene);
+
+         console.log(formatFloat(bb.max.x), formatFloat(bb.min.x),
+         formatFloat(bb.max.y), formatFloat(bb.min.y),
+         formatFloat(bb.max.z), formatFloat(bb.min.z));
+
+         let lines = [];
+         lines.push({ "p": new RC.Vector3(bb.min.x, 0, 0), "c": new RC.Color(1, 0, 0), "text": "x " + formatFloat(bb.min.x) });
+         lines.push({ "p": new RC.Vector3(bb.max.x, 0, 0), "c": new RC.Color(1, 0, 0), "text": "x " + formatFloat(bb.max.x) });
+         lines.push({ "p": new RC.Vector3(0, bb.min.y, 0), "c": new RC.Color(0, 1, 0), "text": "y " + formatFloat(bb.min.y) });
+         lines.push({ "p": new RC.Vector3(0, bb.max.y, 0), "c": new RC.Color(0, 1, 0), "text": "y " + formatFloat(bb.max.y) });
+         if (this.controller.isEveCameraPerspective()) {
+            lines.push({ "p": new RC.Vector3(0, 0, bb.min.z), "c": new RC.Color(0, 0, 1), "text": "z " + formatFloat(bb.min.z) });
+            lines.push({ "p": new RC.Vector3(0, 0, bb.max.z), "c": new RC.Color(0, 0, 1), "text": "z " + formatFloat(bb.max.z) });
+         }
+
+         for (const ax of lines) {
+            let geom = new RC.Geometry();
+            let buf = new Float32Array([0, 0, 0, ax.p.x, ax.p.y, ax.p.z]);
+            geom.vertices = new RC.Float32Attribute(buf, 3);
+            let ss = this.creator.RcMakeStripes(geom, 2, ax.c);
+            this.axis.add(ss);
+         }
+
+         let url_base = this.eve_path + 'sdf-fonts/LiberationSerif-Regular';
+         this.tex_cache.deliver_font(url_base,
+            (texture, font_metrics) => {
+               let diag = new RC.Vector3;
+               bb.getSize(diag);
+               diag = diag.length() / 100;
+               let ag = this.axis;
+               for (const ax of lines) {
+                  const text = new RC.ZText({
+                     text: ax.text,
+                     fontTexture: texture,
+                     xPos: 0.0,
+                     yPos: 0.0,
+                     fontSize: 0.01,
+                     mode: RC.TEXT2D_SPACE_MIXED,
+                     fontHinting: 1.0,
+                     color: this.fgCol,
+                     font: font_metrics,
+                  });
+                  text.position = ax.p;
+                  text.material.side = RC.FRONT_SIDE;
+                  ag.add(text);
+               }
+            },
+            (img) => RC.ZText.createDefaultTexture(img)
+         );
+      };
+
       //==============================================================================
 
       request_render()
@@ -527,9 +670,21 @@ sap.ui.define([
 
          this.rqt.render_main_and_blend_outline();
 
+         // XXXX here add rendering of overlay, e.g.:
+         if (this.rqt.queue.used_fail_count == 0 && this.overlay_scene.children.length > 0) {
+            this.rqt.render_overlay_and_blend_it();
+         }
+         // YYYY Or, it might be better to render overlay after the tone-mapping.
+         // Eventually, if only overlay changes, we don't need to render the base-scene but
+         // only overlay and re-merge them. Need to keep base textures alive in RendeQuTor.
+         // Note that rgt.render_end() releases all std textures.
+
          if (this.rqt.queue.used_fail_count == 0) {
+            // AMT: All render passess are drawn with the black bg
+            //      except of the tone map render pass
+            this.renderer.clearColor = '#' +  this.bgCol.getHexString() + '00';
             this.rqt.render_tone_map_to_screen();
-            // this.rqt.render_final_to_screen();
+            this.renderer.clearColor = "#00000000";
          }
 
          this.rqt.render_end();
@@ -554,6 +709,8 @@ sap.ui.define([
 
          let state = this.rqt.pick(x, y, detect_depth);
 
+         console.log("pick state", state);
+
          if (state.object === null) {
             this.rqt.pick_end();
             return null;
@@ -581,9 +738,50 @@ sap.ui.define([
             ctrl_obj = ctrl_obj.parent;
 
          state.ctrl = ctrl_obj.get_ctrl(ctrl_obj, top_obj);
-
-         // console.log("pick result", state);
          return state;
+
+      }
+
+      render_for_Overlay_picking(x, y, detect_depth)
+      {
+         // console.log("RENDER FOR PICKING", this.scene, this.camera, this.canvas, this.renderer);
+
+         if (this.canvas.width <= 0 || this.canvas.height <= 0) return null;
+
+         this.rqt.pick_begin(x, y);
+
+         let state_overlay = this.rqt.pick_overlay(x, y, detect_depth);
+
+         console.log("Overlay pick state", state_overlay);
+
+         if (state_overlay.object === null) {
+            this.rqt.pick_end();
+            return null;
+         }
+
+         let top_obj = state_overlay.object;
+            while (top_obj.eve_el === undefined)
+               top_obj = top_obj.parent;
+
+            state_overlay.top_object = top_obj;
+            state_overlay.eve_el = top_obj.eve_el;
+
+            if (state_overlay.eve_el.fSecondarySelect)
+               this.rqt.pick_instance_overlay(state_overlay);
+
+            this.rqt.pick_end();
+
+            state_overlay.w = this.canvas.width;
+            state_overlay.h = this.canvas.height;
+            state_overlay.mouse = new RC.Vector2( ((x + 0.5) / state_overlay.w) * 2 - 1,
+                                         -((y + 0.5) / state_overlay.h) * 2 + 1 );
+
+            let ctrl_obj = state_overlay.object;
+            while (ctrl_obj.get_ctrl === undefined)
+               ctrl_obj = ctrl_obj.parent;
+
+            state_overlay.ctrl = ctrl_obj.get_ctrl(ctrl_obj, top_obj);
+            return state_overlay;
       }
 
       //==============================================================================
@@ -681,7 +879,7 @@ sap.ui.define([
          let c = pstate.ctrl;
          let idx = c.extractIndex(pstate.instance);
 
-         c.elementHighlighted(idx, null);
+         c.elementHighlighted(idx, null, pstate.object);
 
          if (this.highlighted_top_object !== pstate.top_object)
          {
@@ -818,13 +1016,94 @@ sap.ui.define([
 
          if (pstate) {
             let c = pstate.ctrl;
-            c.elementSelected(c.extractIndex(pstate.instance), event);
+            c.elementSelected(c.extractIndex(pstate.instance), event, pstate.object);
             // WHY ??? this.highlighted_scene = pstate.top_object.scene;
          } else {
             // XXXX HACK - handlersMIR senders should really be in the mgr
 
             this.controller.created_scenes[0].processElementSelected(null, [], event);
          }
+
+
+      }
+
+      handleOverlayMouseUp()
+      {
+         console.log("handleOverlayMouseUp");
+         if(this.firstMouseDown == false)
+         {
+            this.firstMouseDown = true;
+            //this.overlay_scene.children[0].children[0].setNewPositionOffset(this.lastOffsetX, this.lastOffsetY);
+            this.pickedOverlayObj.setNewPositionOffset(this.lastOffsetX, this.lastOffsetY);
+            this.lastOffsetX = 0;
+            this.lastOffsetY = 0;
+            this.initialMouseX = 0;
+            this.initialMouseY = 0;
+            this.scale = false;
+            this.initialSize = 0;
+            this.controls.enablePan = true;
+            this.controls.enableRotate = true;
+         }
+      }
+
+      handleOverlayMouseDown(event)
+      {
+         console.log("handleOverlayMouseDown");
+         let x = event.offsetX * this.canvas.pixelRatio;
+         let y = event.offsetY * this.canvas.pixelRatio;
+         let overlay_pstate = this.render_for_Overlay_picking(x, y, false);
+
+
+
+         if(this.firstMouseDown && overlay_pstate)
+         {
+             this.initialMouseX = x;
+             this.initialMouseY = y;
+             //let c = overlay_pstate.ctrl;
+             this.pickedOverlayObj = overlay_pstate.object;
+             this.firstMouseDown = false;
+
+             if(event.button == 2)
+             {
+               this.scale = true;
+               this.controls.enablePan = false;
+               //this.initialSize = this.overlay_scene.children[0].children[0].fontSize;
+               this.initialSize = this.pickedOverlayObj.fontSize;
+             }
+             else
+               this.controls.enableRotate = false;
+
+         }
+      }
+
+      handleOverlayMouseMove(event)
+      {
+         //console.log("handleOverlayMouseMove");
+
+         if(!this.firstMouseDown)
+         {
+            let x = event.offsetX * this.canvas.pixelRatio;
+            let y = event.offsetY * this.canvas.pixelRatio;
+
+            if(!this.scale)
+            {
+               this.lastOffsetX = (x - this.initialMouseX)/this.canvas.width;
+               this.lastOffsetY = (this.initialMouseY - y)/this.canvas.height;
+               //this.overlay_scene.children[0].children[0].setOffset([this.lastOffsetX, this.lastOffsetY]);
+               this.pickedOverlayObj.setOffset([this.lastOffsetX, this.lastOffsetY]);
+
+            }
+            else
+            {
+               //this.overlay_scene.children[0].children[0].fontSize = this.initialSize + (x - this.initialMouseX);
+               this.pickedOverlayObj.fontSize = this.initialSize + (x - this.initialMouseX);
+
+            }
+            this.render();
+
+
+
+         }
       }
 
       timeStampAttributesAndTextures() {
@@ -881,10 +1160,10 @@ sap.ui.define([
    }</script>
 </head>
 <body>
-   <form name="myform"> 
+   <form name="myform">
       <input type="text" id="filename" name="Filename" value="programs${extra}">
       <input type="button" onClick="save();" value="Save">
-   </form> 
+   </form>
    <pre> ${json} </pre>
 </body>
          </html>`);
